-- =============================================
-- Author:		Produmex
-- Create date: 
-- Description:	Goods Receipt Label
-- =============================================
ALTER PROCEDURE [dbo].[PMX_SP_DirkRaes_GoodsReceiptLabel] 
	-- Parameters for the stored procedure
	@luid int = 0,
	@sscc nvarchar (18),
	@itemCode nvarchar (20),
	@quantity numeric (19,6),
	@quantityUom2 numeric (19,6), --Temp ADDED FOR ISSUE ON Printing Goods Reciept in PDMX
	@bestBeforeDate datetime,
	@batchNumber nvarchar (32),
	@purchaseDocObjType nvarchar(20),
	@purchaseDocEntry int,
	@cardCode nvarchar (15),
	@packagingTypeText nvarchar(max),
	@quantityByPackagingTypeText nvarchar(max),
	@locationCode nvarchar (50),
	@batchNumber2 nvarchar (32),
	@internalLuid int = 0

AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.

	SET NOCOUNT ON;
	SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;

	--Declare return variables
	DECLARE @returnSSCC [nvarchar](18)
	DECLARE @returnGTIN AS [nvarchar] (16) 
	DECLARE @returnLogisticCarrierCode AS [nvarchar] (20)  
	DECLARE @returnItemCode AS [nvarchar](20) 
	DECLARE @returnProductDescription AS [nvarchar](100) 
	DECLARE @returnQuantity AS [numeric](19,0) 
	DECLARE @returnIsLogisticUnit AS [nvarchar](1) 
	DECLARE @returnBatchNumber AS [nvarchar](32) 
	DECLARE @returnBestBeforeDate AS [datetime]
	DECLARE @returnSSCCBarcode AS [nvarchar] (100)
	DECLARE @returnSSCCBarcodeDigit AS [nvarchar] (100)
	DECLARE @returnGTINBarcode AS [nvarchar] (200)
	DECLARE @returnGTINBarcodeDigit AS [nvarchar] (200)
 	DECLARE @returnBarcodeType AS [nvarchar] (1)
	DECLARE @returnGTINLabel AS [nvarchar] (20)
	DECLARE @returnPackagingTypeText AS [nvarchar] (max)
	DECLARE @returnQuantityByPackagingTypeText AS [nvarchar] (max)
	DECLARE @returnSupplierLogisticUnitID AS [nvarchar](100)
	DECLARE @returnZoneTypeCode AS [nvarchar] (50)
	DECLARE @returnBatchNumber2 AS [nvarchar] (36)
	DECLARE @returnWeight AS [numeric](19,0) 
	DECLARE @returnWeightUoM AS [nvarchar] (5)
	

	--Getting the PO info
	--Declare variables for the cursor
	DECLARE @PONumber AS [int]
	DECLARE @SupplierName AS [nvarchar] (100) 
	DECLARE @SupplierAddress AS [nvarchar] (254)  
	DECLARE @ReceptionDate AS [datetime] 


	--Declare variables for product
	DECLARE @GTIN AS [nvarchar] (16) 
	DECLARE @LogisticCarrierCode AS [nvarchar] (20)  
	DECLARE @ProductDescription AS [nvarchar](100) 
	DECLARE @IsLogisticCarrier AS [nvarchar](1) 
	DECLARE @BarcodeType AS [nvarchar] (1)
	DECLARE @TempPackagingTypeText AS [nvarchar] (max)
	DECLARE @TempQuantityByPackagingTypeText AS [nvarchar] (max)
	DECLARE @ZoneTypeCode AS [nvarchar] (50)



	IF(@luid <> 0) BEGIN
	
		--Declare memory table for storing the products on an sscc
		DECLARE @productsTable TABLE (
			[SSCC] [nvarchar] (18) NOT NULL ,
			[GTIN] [nvarchar] (16) NULL ,
			[LogisticCarrierCode] [nvarchar] (20) NULL ,
			[ItemCode] [nvarchar](20) NULL ,
			[ProductDescription] [nvarchar](100) NULL,
			[Quantity] [numeric](19,6) NULL,
			[IsLogisticCarrier] [nvarchar](1) NULL,
			[BatchNumber] [nvarchar](32) NULL,
			[BestBeforeDate] [datetime] NULL,
			[BarcodeType] [nvarchar](1) NULL,
			[PackagingTypesText] nvarchar(max) NULL,
			[QuantityByPackagingTypeText] nvarchar(max) NULL,
			[ZoneTypeCode] [nvarchar] (50) NULL,
			[BatchNumber2] [nvarchar] (36) NULL
		)

		-- Insert the products into an in memory table
		INSERT into @productsTable
			SELECT SSCC, Barcode, LogisticCarrierCode, PMX_SSCC_GOODSRECEIPT_LABEL.ItemCode, ISNULL(ProductDescription, 
			PMX_SSCC_GOODSRECEIPT_LABEL.ItemCode) AS ProductDescription, Quantity , IsLogisticCarrier, BatchNumber, BestBeforeDate,
			ISNULL(U_PMX_BCTY,'G') AS BarcodeType, dbo.PMX_FN_GetPackagingTypesText(PMX_SSCC_GOODSRECEIPT_LABEL.ItemCode, PMX_SSCC_GOODSRECEIPT_LABEL.ItemTransactionalInfoKey, PMX_SSCC_GOODSRECEIPT_LABEL.Quantity, 'B'),
			dbo.PMX_FN_GetQuantityByPackagingTypesText(PMX_SSCC_GOODSRECEIPT_LABEL.ItemCode, PMX_SSCC_GOODSRECEIPT_LABEL.ItemTransactionalInfoKey),PMX_ITZT.ZoneTypeCode,PMX_SSCC_GOODSRECEIPT_LABEL.InternalBatchNumber					
			FROM dbo.PMX_SSCC_GOODSRECEIPT_LABEL 
			INNER JOIN dbo.OITM ON dbo.OITM.ItemCode = dbo.PMX_SSCC_GOODSRECEIPT_LABEL.ItemCode
			LEFT OUTER JOIN PMX_ITZT ON OITM.ItemCode = PMX_ITZT.ItemCode
			WHERE LogUnitIdentKey = @luid
			ORDER BY IsLogisticCarrier, ItemCode

		--Loop tru all the lines
		DECLARE productsCursor CURSOR FOR SELECT SSCC, GTIN, LogisticCarrierCode, ItemCode, ProductDescription, Quantity , IsLogisticCarrier, BatchNumber, BestBeforeDate, BarcodeType, PackagingTypesText, QuantityByPackagingTypeText,ZoneTypeCode,BatchNumber2 FROM @productsTable
		OPEN productsCursor
		FETCH NEXT FROM productsCursor INTO @SSCC, @GTIN, @LogisticCarrierCode, @ItemCode, @ProductDescription, @Quantity , @IsLogisticCarrier, @BatchNumber, @BestBeforeDate, @BarcodeType, @TempPackagingTypeText, @TempQuantityByPackagingTypeText,@ZoneTypeCode,@batchNumber2
		IF(@@FETCH_STATUS >= 0)
		BEGIN
			SET @returnSSCC = @SSCC
			SET @returnGTIN = @GTIN
			SET @returnLogisticCarrierCode = @LogisticCarrierCode
			SET @returnItemCode = @ItemCode
			SET @returnProductDescription = @ProductDescription
			SET @returnQuantity = @Quantity
			SET @returnIsLogisticUnit = @IsLogisticCarrier
			SET @returnBatchNumber = @BatchNumber
			SET @returnBestBeforeDate = @BestBeforeDate
			SET @returnBarcodeType = @BarcodeType
			SET @returnPackagingTypeText = @TempPackagingTypeText
			SET @returnQuantityByPackagingTypeText = @TempQuantityByPackagingTypeText
			SET @returnZoneTypeCode = @ZoneTypeCode
			SET @returnBatchNumber2 = @batchNumber2

			FETCH NEXT FROM productsCursor INTO @SSCC, @GTIN, @LogisticCarrierCode, @ItemCode, @ProductDescription, @Quantity , @IsLogisticCarrier, @BatchNumber, @BestBeforeDate, @BarcodeType, @TempPackagingTypeText, @TempQuantityByPackagingTypeText,@ZoneTypeCode,@batchNumber2
		END
		WHILE @@FETCH_STATUS >= 0 BEGIN
			
			--Only when the previous product is the same as product, 
			--or if the current product is a logistic carrier and the quantity of the logistic carrier is 1, we can keep the product description, count and gtin.
			--In all other cases, we need to remove the product and count
			IF(@returnItemCode = @ItemCode OR
				(@ItemCode = @LogisticCarrierCode AND
				@Quantity = 1))
			BEGIN
				--Only when the current product is a logistic carrier and 
				--the quantity of the logistic carrier is 1, we can keep the product batch and best before date.
				--In all other cases, we need to remove the product
				IF(NOT(@ItemCode = @LogisticCarrierCode AND
					@Quantity = 1))
				BEGIN
					IF(@BatchNumber <> @returnBatchNumber) SET @returnBatchNumber = NULL
					IF(@BestBeforeDate <> @returnBestBeforeDate) SET @returnBestBeforeDate = NULL
				END
				
				IF(@returnQuantity IS NOT NULL)
				BEGIN
					--if we can add quantity, we check if the product is the logistic carrier.
					--if so, we add the new quantity, but we deduct 1 from the quantity because we don't count the logistic carrier
					IF(@ItemCode = @LogisticCarrierCode) SET @returnQuantity = @returnQuantity + (@Quantity - 1)
					ELSE SET @returnQuantity = @returnQuantity + @Quantity
				END
			END
			ELSE
			BEGIN
				SET @returnItemCode = NULL
				SET @returnGTIN = NULL
				SET @returnProductDescription = NULL
				SET @returnQuantity = NULL
				SET @returnBatchNumber = NULL
				SET @returnBestBeforeDate = NULL
				SET @returnBarcodeType = NULL
				SET @returnPackagingTypeText = NULL
				SET @returnQuantityByPackagingTypeText = NULL
				SET @returnZoneTypeCode = NULL
				SET @returnBatchNumber2 = NULL
			END

			FETCH NEXT FROM productsCursor INTO @SSCC, @GTIN, @LogisticCarrierCode, @ItemCode, @ProductDescription, @Quantity , @IsLogisticCarrier, @BatchNumber, @BestBeforeDate, @BarcodeType, @TempPackagingTypeText, @TempQuantityByPackagingTypeText,@ZoneTypeCode,@batchNumber2
		END

		CLOSE productsCursor 
		DEALLOCATE productsCursor

	--Set the barcodes
		SET @returnSSCCBarcode = dbo.PMX_FN_GenerateGS1Barcode( @returnSSCC, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, '' )
		SET @returnSSCCBarcodeDigit = dbo.PMX_FN_GenerateGS1BarcodeDigit ( @returnSSCC, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL )
		SET @returnGTINBarcode = dbo.PMX_FN_GenerateGS1Barcode( NULL, @returnGTIN, @returnIsLogisticUnit, @returnBarcodeType,@returnBestBeforeDate, @returnBatchNumber, @returnQuantity, NULL, NULL, NULL, '')
		SET @returnGTINBarcodeDigit = dbo.PMX_FN_GenerateGS1BarcodeDigit ( NULL , @returnGTIN, @returnIsLogisticUnit, @returnBarcodeType,@returnBestBeforeDate, @returnBatchNumber, @returnQuantity, NULL, NULL, NULL)


		IF(@returnIsLogisticUnit = 'Y') SET @returnGTINLabel = 'GTIN'
		ELSE SET @returnGTINLabel = 'CONTENT'


		DECLARE poCursor CURSOR FOR SELECT
			PurchaseDocNum,
			SupplierName,
			SupplierAddress,
			ReceptionDate
		FROM dbo.PMX_DELIVERY_INFO_LABEL
		WHERE LogUnitIdentKey = @luid
		ORDER BY PurchaseDocNum DESC

		OPEN poCursor
		FETCH NEXT FROM poCursor INTO @PONumber, @SupplierName, @SupplierAddress, @ReceptionDate

		CLOSE poCursor 
		DEALLOCATE poCursor


DECLARE SupplierPalletCursor CURSOR FOR SELECT
	SupplierLogisticUnitID
	FROM dbo.PMX_LUID
	WHERE internalkey = @luid
OPEN SupplierPalletCursor
FETCH NEXT FROM SupplierPalletCursor INTO @returnSupplierLogisticUnitID
CLOSE SupplierPalletCursor
DEALLOCATE SupplierPalletCursor

	END
	ELSE
	BEGIN
		SET @returnSSCC = @SSCC
		SET @returnItemCode = @ItemCode
		SET @returnQuantity = @Quantity
		SET @returnBatchNumber = @BatchNumber
		SET @returnBestBeforeDate = @BestBeforeDate
		SET @returnPackagingTypeText = @PackagingTypeText
		SET @returnQuantityByPackagingTypeText = @quantityByPackagingTypeText
		SET @returnBatchNumber2 =@batchNumber2

		SET @ReceptionDate = Getdate()

		--Purchase info
		DECLARE poCursor CURSOR FOR SELECT
			DocNum,
			CardName,
			[Address]
		FROM dbo.OPOR
		WHERE DocEntry = @purchaseDocEntry
		ORDER BY DocNum DESC

		OPEN poCursor
		FETCH NEXT FROM poCursor INTO @PONumber, @SupplierName, @SupplierAddress

		CLOSE poCursor 
		DEALLOCATE poCursor

		SELECT @returnIsLogisticUnit = OITM.U_PMX_LOUN, @returnBarcodeType = ISNULL(U_PMX_BCTY,'G'), @returnGTIN = CodeBars, @returnProductDescription = ItemName FROM OITM WHERE OITM.ItemCode = @itemCode

		IF(@returnIsLogisticUnit = 'Y') SET @returnGTINLabel = 'GTIN'
		ELSE SET @returnGTINLabel = 'CONTENT'

	--Set the barcodes
		SET @returnSSCCBarcode = dbo.PMX_FN_GenerateGS1Barcode( @returnSSCC, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, '' )
		SET @returnSSCCBarcodeDigit = dbo.PMX_FN_GenerateGS1BarcodeDigit ( @returnSSCC, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL )
		SET @returnGTINBarcode = dbo.PMX_FN_GenerateGS1Barcode( NULL, @returnGTIN, @returnIsLogisticUnit, @returnBarcodeType,@returnBestBeforeDate, @returnBatchNumber, @returnQuantity, NULL, NULL, NULL, '')
		SET @returnGTINBarcodeDigit = dbo.PMX_FN_GenerateGS1BarcodeDigit ( NULL , @returnGTIN, @returnIsLogisticUnit, @returnBarcodeType,@returnBestBeforeDate, @returnBatchNumber, @returnQuantity, NULL, NULL, NULL)


		DECLARE SupplierPalletCursor CURSOR FOR SELECT
			SupplierLogisticUnitID
			FROM dbo.PMX_LUID
			WHERE internalkey = @internalLuid
		OPEN SupplierPalletCursor
		FETCH NEXT FROM SupplierPalletCursor INTO @returnSupplierLogisticUnitID
		CLOSE SupplierPalletCursor
		DEALLOCATE SupplierPalletCursor

	 ---- Getting the zone types -----
	 SELECT TOP 1   @returnZoneTypeCode =  PMX_ITZT.ZoneTypeCode  from PMX_ITZT where PMX_ITZT.ItemCode = @itemCode



	 -----End getting the zone types -----
	  ---- Get Weight for Label-----
	 --This is not the actual weight recorded on the Goods Reciept. Agreed with Fred to calculate it based on the item master value. Aware that if value is changed, a reprint will give a different value
		 SELECT @returnWeight = @returnQuantity * BWeight1, 
		@returnWeightUoM = (CASE BWght1Unit WHEN 2 THEN 'g' WHEN 3 THEN 'kg' ELSE '' END) FROM OITM WHERE ItemCode = @itemCode
	 ---- END Get Weight for Label-----

END


	--Return the values
	SELECT	@returnSSCC AS SSCC, 	
			ISNULL(@returnGTIN,'N/A') AS GTIN,
			ISNULL(@returnProductDescription,'N/A') AS ProductDescription, 
			ISNULL(CAST (@returnQuantity AS nvarchar),'N/A') Quantity,
			ISNULL(@returnBatchNumber,'N/A') AS BatchNumber, 
			ISNULL( CONVERT(nvarchar, @returnBestBeforeDate, 103),'N/A')  AS BestBeforeDate,
			@returnSSCCBarcode AS SSCCBarcode,
			@returnSSCCBarcodeDigit AS SSCCBarcodeDigit,
			@returnGTINBarcode AS GTINBarcode,
			@returnGTINBarcodeDigit AS GTINBarcodeDigit,
			ISNULL( CAST( @PONumber AS nvarchar),'N/A') AS PONumber,
			ISNULL(@SupplierName,'N/A') AS SupplierName,
			ISNULL(@SupplierAddress,'N/A') AS SupplierAddress,
			ISNULL( CONVERT(nvarchar, @ReceptionDate, 103),'N/A') AS ReceptionDate,
			@returnGTINLabel AS GTINLabel,
			@returnPackagingTypeText AS PackagingTypeText,
			@returnQuantityByPackagingTypeText AS QuantityByPackagingTypeText,
			@returnSupplierLogisticUnitID AS SupplierPallet,
			@returnitemcode AS ItemCode,
			@returnZoneTypeCode As ZoneTypeCode,
			@returnBatchNumber2 AS BatchNumber2,
			ISNULL(@returnWeight,0)AS [Weight],
			ISNULL(@returnWeightUoM,'') AS [WeighUoM]

	
END
