ALTER PROCEDURE [dbo].[PMX_SP_MoviantoColliPackageLabel_V2]
	-- Add the parameters for the stored procedure here
	@PickListDocEntry int = 0,
	@SalesOrderDocEntry int = 0
AS
BEGIN
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	--VERSION 03OCT2016 - TOAN

	/*The way that the query uses the PMX_MOV_PLCD table  is because they want to print the packaging before they make the delivery in SAP.  Unfortunately this causes the issue that when more than one colli is used that every line in the order is multiplied.
	We work around this by grouping in the procedure or in the crystal report by using the grouping and the suppress filters. But there is a risk which the customer is aware of with the column In Box that there is not a guarantee that it will always be correct. 
	Whilst for all of our testing it has worked so far, It could end up being in a different colli. The risk is limited, but it could happen. PROD-2536*/
	/*
	KNOWN LAYOUTS USING THIS STORED PROCEDURE:
		10xGenomics_DN.rpt
		10xGenomics_INVOICE.rpt
		Accelerate_DN.rpt
		Lamepro_DN.rpt
		MoviantoColliPackageLabel.rpt
		MoviantoColliPackageLabel_new.rpt
		SpeedX_DN.rpt
	*/
	
	SET NOCOUNT ON;
	SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED;

	DECLARE @CardCode NVARCHAR(MAX)
	SELECT @CardCode = U_PMX_DR_IMSC FROM ORDR WHERE DocEntry = @SalesOrderDocEntry

	IF @CardCode IN ('120006','101009','112110','119007')  --10x,Accelerate,Lamepro,SpeedX
	BEGIN
	SELECT 	DISTINCT
		PMX_MOV_PLCD.InternalKey, PMX_MOV_PLCD.PickListDocEntry, PMX_MOV_PLCD.SalesOrderDocEntry,convert(date,PMX_MOV_PLCD.CreateDateTime,106) AS CreateDate, PMX_MOV_PLCD.ColliID, PMX_MOV_PLCD.Carrier,
			PMX_MOV_PLCD.Width, PMX_MOV_PLCD.Length, PMX_MOV_PLCD.Height, PMX_MOV_PLCD.Weight, PMX_MOV_PLCD.PalletID ,
			T0.CardName,
			T0.StreetS,
			T0.CityS,
			T0.ZipCodeS,
			T0.CountryS,
			PMX_MOV_PLCD.AWBNr,
			T0.U_PMX_MOV_TXT01,
			T0.U_PMX_MOV_TXT02,
			T0.U_PMX_MOV_TXT03,
			T0.U_PMX_MOV_TXT04,
			T0.U_PMX_MOV_TXT06,
			T0.U_PMX_MOV_TXT07,
			T0.U_PMX_MOV_TXT08,
			T0.U_PMX_MOV_TXT09,
			T0.U_PMX_MOV_TXT10,
			T0.NumAtCard,
			T0.DocDueDate,
			T0.U_CUSTOMER_PO,
			T0.U_PMX_DR_SAL_ORDR,
			T0.Comments,
			T0.U_PMX_DR_NAM2,
			T0.U_PMX_DR_ADR2,
			T0.U_PMX_DR_EREM,
			T0.TrnspCode,
			T0.PickRmrk,
			T0.U_PMX_DR_COMMENTS,
			ISNULL(PMX_INVT.LogUnitIdentKey, PMX_INVD.LogUnitIdentKey )AS LUID,
			T0.LineNum,
			(CASE WHEN T0.TreeType = 'S' THEN T0.ItemCode ELSE ISNULL(PMX_INVT.ItemCode,PMX_INVD.ItemCode) END) ItemCode,
			T0.TreeType,
			(CASE WHEN T0.TreeType = 'S' THEN T0.ItemName ELSE OITM.ItemName END) ItemName,
			(CASE WHEN T0.TreeType = 'S' THEN T0.Quantity ELSE (CASE WHEN ISNULL(PMX_INVT.Quantity,PMX_INVD.Quantity*-1) > T0.Quantity THEN T0.Quantity ELSE ISNULL(PMX_INVT.Quantity,PMX_INVD.Quantity*-1) END) END) Quantity,
			T0.Price,
			T0.DiscPrcnt,
			T0.PriceAfterDiscount,
			PMX_ITRI.BatchNumber AS Batchnumber,
			PMX_ITRI.BestBeforeDate AS BestBeforeDate,
			T0.LanguageCode,
			T0.Class,
			T0.CountryOfOrigin,
			T0.EQ,
			T0.HSCODE,
			T0.LABEL,
			T0.Liquid_Solid,
			T0.LQ,
			T0.MSDS,
			T0.NetWeightVKE,
			T0.NetWeightSKU,
			T0.VolumeVKE,
			T0.VolumeSKU,
			T0.PackingGroup,
			T0.Freetxt01,
			T0.Freetxt02,
			T0.Freetxt03,
			T0.Freetxt04,
			T0.Freetxt05,
			T0.Freetxt06,
			OITM.QryGroup1 AS Property01,
			OITM.QryGroup2 AS Property02,
			OITM.QryGroup3 AS Property03,
			OITM.QryGroup4 AS Property04,
			OITM.QryGroup5 AS Property05,
			OITM.QryGroup6 AS Property06,
			OITM.QryGroup7 AS Property07,
			OITM.QryGroup8 AS Property08,
			OITM.QryGroup9 AS Property09,
			OITM.QryGroup10 AS Property10

	FROM  PMX_MOV_PLCD 
	INNER JOIN PMX_MoviantoSalesOrder T0 ON T0.DocEntry = PMX_MOV_PLCD.SalesOrderDocEntry
	LEFT JOIN PMX_INVT ON RIGHT(PMX_MOV_PLCD.ColliID,18) = RIGHT (PMX_INVT.SSCC,18) AND PMX_INVT.ItemCode = T0.ItemCode
	LEFT JOIN PMX_INVD ON RIGHT(PMX_MOV_PLCD.ColliID,18) = RIGHT (PMX_INVD.SSCC,18) AND PMX_INVD.ItemCode = T0.ItemCode AND PMX_INVD.TransType = '15'
	LEFT JOIN PMX_ITRI ON ISNULL(PMX_INVT.ItemTransactionalInfoKey,PMX_INVD.ItemTransactionalInfoKey) = PMX_ITRI.InternalKey
	LEFT JOIN OITM ON ISNULL(PMX_INVT.ItemCode,PMX_INVD.ItemCode) = OITM.ItemCode
	WHERE  PMX_MOV_PLCD.PickListDocEntry = @PickListDocEntry and PMX_MOV_PLCD.SalesOrderDocEntry = @SalesOrderDocEntry
	AND ISNULL((CASE WHEN T0.TreeType = 'S' THEN T0.ItemCode ELSE ISNULL(PMX_INVT.ItemCode,PMX_INVD.ItemCode) END),'') <> ''

	ORDER BY T0.LineNum

	END
	ELSE -- ALL
	BEGIN
	
    -- Insert statements for procedure here
	select PMX_MOV_PLCD.InternalKey, PMX_MOV_PLCD.PickListDocEntry, PMX_MOV_PLCD.SalesOrderDocEntry,convert(date,PMX_MOV_PLCD.CreateDateTime,106) AS CreateDate, PMX_MOV_PLCD.ColliID, PMX_MOV_PLCD.Carrier,
	PMX_MOV_PLCD.Width, PMX_MOV_PLCD.Length, PMX_MOV_PLCD.Height, PMX_MOV_PLCD.Weight, PMX_MOV_PLCD.PalletID ,
	ORDR.CardName,
	RDR12.StreetS,
	RDR12.CityS,
	RDR12.ZipCodeS,
	RDR12.CountryS,
	PMX_MOV_PLCD.AWBNr,
	ORDR.U_PMX_MOV_TXT01,
	ORDR.U_PMX_MOV_TXT02,
	ORDR.U_PMX_MOV_TXT03,
	ORDR.U_PMX_MOV_TXT04,
	ORDR.U_PMX_MOV_TXT06,
	ORDR.U_PMX_MOV_TXT07,
	ORDR.U_PMX_MOV_TXT08,
	ORDR.U_PMX_MOV_TXT09,
	ORDR.U_PMX_MOV_TXT10,
	ORDR.NumAtCard,
	ORDR.DocDueDate,
	ORDR.U_CUSTOMER_PO,
	ORDR.U_PMX_DR_SAL_ORDR,
	ORDR.Comments,
	ORDR.U_PMX_DR_NAM2,
	ORDR.U_PMX_DR_ADR2,
	ORDR.U_PMX_DR_EREM,
	ORDR.TrnspCode,
	ORDR.PickRmrk,
	convert(varchar(100),RDR1.U_PMX_DR_COMMENTS) AS U_PMX_DR_COMMENTS,
	ORDR.U_PMX_MOV_TXT05 AS LanguageCode,
		PMX_INVT.LogUnitIdentKey AS LUID,
	PMX_INVT.ItemCode,
		OITM.TreeType,
	OITM.ItemName,
	PMX_INVT.Quantity,
	PMX_ITRI.BatchNumber AS Batchnumber,
	PMX_ITRI.BestBeforeDate AS BestBeforeDate,
	ORDR.U_PMX_MOV_TXT05 AS LanguageCode,
OITM.U_PMX_DG_CLASS AS Class,
OITM.U_PMX_DG_CTRYOFORG AS CountryOfOrigin,
OITM.U_PMX_DG_EQ AS EQ,
OITM.U_PMX_DG_HSCODE AS HSCODE,
OITM.U_PMX_DG_LABEL AS LABEL,
OITM.U_PMX_DG_LIQSOL AS Liquid_Solid,
OITM.U_PMX_DG_LQ AS LQ,
OITM.U_PMX_DG_MSDS AS MSDS,
OITM.U_PMX_DG_NWKVE AS NetWeightVKE,
OITM.U_PMX_DG_NWSKU AS NetWeightSKU,
OITM.U_PMX_DG_VOLKVE AS VolumeVKE,
OITM.U_PMX_DG_VOLSKU AS VolumeSKU,
OITM.U_PMX_DG_PKGGRP AS PackingGroup,
OITM.U_PMX_DG_TXT01 AS Freetxt01,
OITM.U_PMX_DG_TXT02 AS Freetxt02,
OITM.U_PMX_DG_TXT03 AS Freetxt03,
OITM.U_PMX_DG_TXT04 AS Freetxt04,
OITM.U_PMX_DG_TXT05 AS Freetxt05,
		OITM.U_PMX_DG_TXT06 AS Freetxt06,
		OITM.QryGroup1 AS Property01,
		OITM.QryGroup2 AS Property02,
		OITM.QryGroup3 AS Property03,
		OITM.QryGroup4 AS Property04,
		OITM.QryGroup5 AS Property05,
		OITM.QryGroup6 AS Property06,
		OITM.QryGroup7 AS Property07,
		OITM.QryGroup8 AS Property08,
		OITM.QryGroup9 AS Property09,
		OITM.QryGroup10 AS Property10

	from PMX_MOV_PLCD
		INNER JOIN ORDR ON PMX_MOV_PLCD.SalesOrderDocEntry = ORDR.DocEntry
		INNER JOIN RDR1 ON ORDR.DocEntry = RDR1.DocEntry
		INNER JOIN RDR12 ON ORDR.DocEntry = RDR12.DocEntry
		LEFT JOIN PMX_INVT ON RIGHT(PMX_MOV_PLCD.ColliID,18) = RIGHT (PMX_INVT.SSCC,18)
		LEFT JOIN PMX_ITRI ON PMX_INVT.ItemTransactionalInfoKey = PMX_ITRI.InternalKey
		LEFT JOIN OITM ON PMX_INVT.ItemCode = OITM.ItemCode
	WHERE  PMX_MOV_PLCD.PickListDocEntry = @PickListDocEntry and PMX_MOV_PLCD.SalesOrderDocEntry = @SalesOrderDocEntry

	GROUP BY

	PMX_MOV_PLCD.InternalKey, PMX_MOV_PLCD.PickListDocEntry, PMX_MOV_PLCD.SalesOrderDocEntry,convert(date,PMX_MOV_PLCD.CreateDateTime,106), PMX_MOV_PLCD.ColliID, PMX_MOV_PLCD.Carrier,
	PMX_MOV_PLCD.Width, PMX_MOV_PLCD.Length, PMX_MOV_PLCD.Height, PMX_MOV_PLCD.Weight, PMX_MOV_PLCD.PalletID ,
	ORDR.CardName,
	RDR12.StreetS,
	RDR12.CityS,
	RDR12.ZipCodeS,
	RDR12.CountryS,
	PMX_MOV_PLCD.AWBNr,
	ORDR.U_PMX_MOV_TXT01,
	ORDR.U_PMX_MOV_TXT02,
	ORDR.U_PMX_MOV_TXT03,
	ORDR.U_PMX_MOV_TXT04,
	ORDR.U_PMX_MOV_TXT06,
	ORDR.U_PMX_MOV_TXT07,
	ORDR.U_PMX_MOV_TXT08,
	ORDR.U_PMX_MOV_TXT09,
	ORDR.U_PMX_MOV_TXT10,
	ORDR.NumAtCard,
	ORDR.DocDueDate,
	ORDR.U_CUSTOMER_PO,
	ORDR.U_PMX_DR_SAL_ORDR,
	ORDR.Comments,
	ORDR.U_PMX_DR_NAM2,
	ORDR.U_PMX_DR_ADR2,
	ORDR.U_PMX_DR_EREM,
	ORDR.TrnspCode,
	ORDR.PickRmrk,
	convert(varchar(100),RDR1.U_PMX_DR_COMMENTS ),---AS U_PMX_DRT_COMMENTS,
	ORDR.U_PMX_MOV_TXT05 ,
		PMX_INVT.LogUnitIdentKey,
	PMX_INVT.ItemCode,
	OITM.ItemName,
		OITM.TreeType,
	PMX_INVT.Quantity,
	PMX_ITRI.BatchNumber ,
	PMX_ITRI.BestBeforeDate,
	ORDR.U_PMX_MOV_TXT05 ,
OITM.U_PMX_DG_CLASS ,
OITM.U_PMX_DG_CTRYOFORG ,
OITM.U_PMX_DG_EQ ,
OITM.U_PMX_DG_HSCODE,
OITM.U_PMX_DG_LABEL ,
OITM.U_PMX_DG_LIQSOL ,
OITM.U_PMX_DG_LQ ,
OITM.U_PMX_DG_MSDS ,
OITM.U_PMX_DG_NWKVE ,
OITM.U_PMX_DG_NWSKU ,
OITM.U_PMX_DG_VOLKVE ,
OITM.U_PMX_DG_VOLSKU ,
OITM.U_PMX_DG_PKGGRP ,
OITM.U_PMX_DG_TXT01 ,
OITM.U_PMX_DG_TXT02 ,
OITM.U_PMX_DG_TXT03 ,
OITM.U_PMX_DG_TXT04 ,
OITM.U_PMX_DG_TXT05 ,
		OITM.U_PMX_DG_TXT06,
		OITM.QryGroup1,
		OITM.QryGroup2,
		OITM.QryGroup3,
		OITM.QryGroup4,
		OITM.QryGroup5,
		OITM.QryGroup6,
		OITM.QryGroup7,
		OITM.QryGroup8,
		OITM.QryGroup9,
		OITM.QryGroup10

	END
END