﻿using System;
using System.Globalization;
using System.Reflection;
using System.Text;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.Providers;

public class Script
{
    private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

    public static bool PrintReport(PmxPrintReportEventType eventType, int key, PmxDbConnection dbConn)
    {
        // *** Your script goes here ***

        string query = GetAutomaticShippingTypeForPickList(key);

        using (ISboRecordset rs = SboRecordsetHelper.RunQuery(s_log, query, dbConn))
        {
            if (rs.RecordCount > 0)
            {
                //The picklist is automatic, so do not print label
                return false;
            }
        }

        query = GetOrderInfo(key, "3230", "Y");

        using (ISboRecordset rs = SboRecordsetHelper.RunQuery(s_log, query, dbConn))
        {
            if (rs.RecordCount > 0)
            {

                return false;
            }
        }

        return true;
    }

    private static string GetAutomaticShippingTypeForPickList(int luid)
    {
        return string.Format(@"SELECT     dbo.OSHP.U_PMX_AUSH
        FROM         dbo.PMX_PLHE INNER JOIN
                              dbo.PMX_PLPH ON dbo.PMX_PLHE.PickListProposalEntry = dbo.PMX_PLPH.DocEntry INNER JOIN
                              dbo.PMX_PLPL ON dbo.PMX_PLPH.DocEntry = dbo.PMX_PLPL.DocEntry INNER JOIN
                              dbo.ORDR ON dbo.PMX_PLPL.BaseEntry = dbo.ORDR.DocEntry AND dbo.PMX_PLPL.BaseType = dbo.ORDR.ObjType INNER JOIN
                              dbo.PMX_LPLM ON dbo.PMX_PLHE.DocEntry = dbo.PMX_LPLM.PickListDocEntry INNER JOIN
                              dbo.PMX_MVLI ON dbo.PMX_LPLM.MoveDocEntry = dbo.PMX_MVLI.DocEntry AND 
                              dbo.PMX_LPLM.MoveLineNum = dbo.PMX_MVLI.LineNum LEFT OUTER JOIN
                              dbo.OSHP ON dbo.ORDR.TrnspCode = dbo.OSHP.TrnspCode
        WHERE    dbo.OSHP.U_PMX_AUSH = 'Y' AND (dbo.PMX_MVLI.DestLogUnitIdentKey = {0})", luid.ToString());


    }


    private static string GetOrderInfo(int luid, string opdrachtgever, string isInterfaced)
    {
        return string.Format(@"SELECT     dbo.OSHP.U_PMX_AUSH, dbo.ORDR.U_PMX_DR_IMSC, dbo.ORDR.U_PMX_INFC
        FROM         dbo.PMX_PLHE INNER JOIN
                              dbo.PMX_PLPH ON dbo.PMX_PLHE.PickListProposalEntry = dbo.PMX_PLPH.DocEntry INNER JOIN
                              dbo.PMX_PLPL ON dbo.PMX_PLPH.DocEntry = dbo.PMX_PLPL.DocEntry INNER JOIN
                              dbo.ORDR ON dbo.PMX_PLPL.BaseEntry = dbo.ORDR.DocEntry AND dbo.PMX_PLPL.BaseType = dbo.ORDR.ObjType INNER JOIN
                              dbo.PMX_LPLM ON dbo.PMX_PLHE.DocEntry = dbo.PMX_LPLM.PickListDocEntry INNER JOIN
                              dbo.PMX_MVLI ON dbo.PMX_LPLM.MoveDocEntry = dbo.PMX_MVLI.DocEntry AND 
                              dbo.PMX_LPLM.MoveLineNum = dbo.PMX_MVLI.LineNum LEFT OUTER JOIN
                              dbo.OSHP ON dbo.ORDR.TrnspCode = dbo.OSHP.TrnspCode
        WHERE    (dbo.PMX_MVLI.DestLogUnitIdentKey = {0}) 
		AND dbo.ORDR.U_PMX_DR_IMSC = '{1}' AND dbo.ORDR.U_PMX_INFC = '{2}'", luid.ToString(), opdrachtgever, isInterfaced);


    }
}
