﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Reflection;
using System.Text;
using Produmex.Foundation;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf;
using Produmex.Foundation.Wwf.Sbo;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.MoviantoNL;
using Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Collections;

// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace WorkflowScript_BeforeAutomaticShippingHookScript
{
    /// <summary>
    /// This is a hook script, meant to be customized on a "per project" basis
    /// </summary>
    public class WorkflowScript_BeforeAutomaticShippingHookScript : WorkflowInstanceScriptBase
    {

        private static Collection<string> LIST_OF_OPDRACHTGEVERS = new Collection<string>();

        private static void FillListOfInitiators()
        {
            LIST_OF_OPDRACHTGEVERS.Add("3230");
            LIST_OF_OPDRACHTGEVERS.Add("3230Test");
        }

        private static class Definitions
        {
            //report definitions
            public const string REPORT_PATH_LABEL = @"MoviantoNLShippingLabelColli.rpt";
            public const string REPORT_PATH_SPECIFIC_LABEL = @"MoviantoNLLabel GSK.rpt";
            public const int NUMBER_OF_LABELS = 1;
        }

        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<int> PickListDocEntry;
        public ReadWriteBinder<bool> DoAutomaticShipping;

        // Output parameters * do not change *
        public ReadWriteBinder<PmxDictionary<string, object>> DeliveryData;

        // Sub-flows
        // <none>

        /// <summary>
        /// Gets or sets the delivery data.
        /// </summary>
        /// <value>The delivery data.</value>
        private PmxDictionary<string, object> deliveryData
        {
            get { return DeliveryData.Get(); }
            set { DeliveryData.Set(value); }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_BeforeAutomaticShippingHookScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_BeforeAutomaticShippingHookScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            //initialize return value
            deliveryData = new PmxDictionary<string, object>();

            string initialErrorKey = null;
            Message msg = null;
            double numberOfColliDouble = 0;
            int numberOfColli = 0;

            string query = null;
            DataSet dsItems = null;
            string logisticCarrierCode = null;
            int luid = 0;
            DataSet dsLuid = null;
            DataSet dsOpdrachtgevers = null;

            FillListOfInitiators();


        Step_EnterNumberOfColli:

            #region ENTER NUMBER OF COLLI

            numberOfColli = 0;
            numberOfColliDouble = 0;


            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IEnterQuantityScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
               "InitialErrorKey", initialErrorKey,
               "TitleKey", "MSG_TITLE_ENTER_NUMBER_OF_COLLI",
               "InitialQuantity", 1,
               "NumberOfDigitsAfterDecimal", 0,
               "MaximumQuantity", -1
               ));
            msg = WaitForMessage();

            initialErrorKey = null;


            if (msg.Name.EndsWith(".QuantityEntered"))
            {
                numberOfColliDouble = ExtractParameter<double>(msg.Parameters, "quantity");
                goto Step_SelectLogisticCarrier;
            }

            //not allowed to go back
            initialErrorKey = "MSG_ERROR_CANNOT_GO_BACK";
            goto Step_EnterNumberOfColli;



            #endregion

        Step_SelectLogisticCarrier:

            #region SELECT LOGISTIC CARRIER

            logisticCarrierCode = null;

            //no items in list, so run the query again
            query = BuildQuery.GetProducts(null, null, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit.Sales, true, sboProviderService.GetDbTool());
            dsItems = sboProviderService.RunView(false, null, null, query);

            if (this.GetNumberOfRows(dsItems) > 0)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectProductScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "InitialErrorKey", initialErrorKey,
                        "TitleKey", "MSG_TITLE_SELECT_LOGISTIC_CARRIER",
                        "ProductDS", dsItems
                        ));
                msg = WaitForMessage();

                //clear data
                initialErrorKey = null;

                if (msg.Name.EndsWith(".ProductSelected"))
                {
                    logisticCarrierCode = ExtractParameter<string>(msg.Parameters, "itemCode");

                    //fill return value
                    goto Step_FillReturnValue;
                }

                goto Step_EnterNumberOfColli;
            }
            else
            {
                //No logistic carriers in the system, so only fill in the number of colli
                goto Step_FillReturnValue;
            }

            #endregion

        Step_FillReturnValue:

            #region FILL RETURN VALUE

            numberOfColli = Convert.ToInt32(numberOfColliDouble);
            deliveryData.Add(PmxSalesDeliveryTable.Columns.Colli.NAME, numberOfColli);

            if (!string.IsNullOrEmpty(logisticCarrierCode))
            {
                deliveryData.Add(PmxSalesDeliveryTable.Columns.LogisticCarrierCode.NAME, logisticCarrierCode);
            }

            goto Step_PrintLabel;

            #endregion

        Step_PrintLabel:

            #region PRINT LABEL

            string reportPath = Definitions.REPORT_PATH_LABEL;

            //get the luid
            query = BuildQuery.GetLUID(PickListDocEntry.Get());
            dsLuid = sboProviderService.RunView(false, null, null, query);

            if (this.GetNumberOfRows(dsLuid) > 0)
            {
                luid = Convert.ToInt32(dsLuid.Tables[0].Rows[0][PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME]);
            }

            //check if the specific label needs to be printed
            query = BuildQuery.GetOrderInfo(luid);
            dsOpdrachtgevers = sboProviderService.RunView(false, null, null, query);

            if (this.GetNumberOfRows(dsOpdrachtgevers) > 0)
            {
                //we found data, so now we check if the initiator is in the list
                if( LIST_OF_OPDRACHTGEVERS.Contains( Convert.ToString(dsOpdrachtgevers.Tables[0].Rows[0][PmxSalesDeliveryTable.Columns.InitiatorCode.NAME])))
                {
                    reportPath = Definitions.REPORT_PATH_SPECIFIC_LABEL;
                }
            }


            for (int iLoop = 1; iLoop <= numberOfColli; iLoop++)
            {
                //print label
                sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintLabel), luid, string.Format("{0}/{1}", iLoop, numberOfColli), session.DeviceInstanceId, reportPath);
            }

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
            DefaultCultureInfo.Get(), BuildParamCollection(
                "MessageKey", "MSG_QUESTION_REPRINT_LABEL",
                "TitleKey", "MSG_TITLE_REPRINT_LABEL"));

            msg = WaitForMessage();

            if (msg.Name.EndsWith(".Yes"))
            {
                goto Step_PrintLabel;
            }

            #endregion


        }

        #endregion

        #region HELPERS

        /// <summary>
        /// Executes the print label.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOsePrinter ExecutePrintLabel(PmxDbConnection conn, object[] parameters)
        {
            PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

            Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
            reportParameters.Add(new ReportParameter("@luid", (int)parameters[0]));
            reportParameters.Add(new ReportParameter("@colli", (string)parameters[1]));

            return prov.PrintReport(reportParameters, (string)parameters[3], Definitions.NUMBER_OF_LABELS, (string)parameters[2], null, false);
        }

        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null && ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }
            return 0;
        }
        #endregion


        #region QUERY

        /// <summary>
        /// Static class grouping our native queries
        /// </summary>
        private class BuildQuery
        {

            public static string GetOrderInfo(int luid)
            {
                return string.Format(@"SELECT  DISTINCT dbo.ORDR.U_PMX_DR_IMSC
        FROM         dbo.PMX_PLHE INNER JOIN
                              dbo.PMX_PLPH ON dbo.PMX_PLHE.PickListProposalEntry = dbo.PMX_PLPH.DocEntry INNER JOIN
                              dbo.PMX_PLPL ON dbo.PMX_PLPH.DocEntry = dbo.PMX_PLPL.DocEntry INNER JOIN
                              dbo.ORDR ON dbo.PMX_PLPL.BaseEntry = dbo.ORDR.DocEntry AND dbo.PMX_PLPL.BaseType = dbo.ORDR.ObjType INNER JOIN
                              dbo.PMX_LPLM ON dbo.PMX_PLHE.DocEntry = dbo.PMX_LPLM.PickListDocEntry INNER JOIN
                              dbo.PMX_MVLI ON dbo.PMX_LPLM.MoveDocEntry = dbo.PMX_MVLI.DocEntry AND 
                              dbo.PMX_LPLM.MoveLineNum = dbo.PMX_MVLI.LineNum LEFT OUTER JOIN
                              dbo.OSHP ON dbo.ORDR.TrnspCode = dbo.OSHP.TrnspCode
        WHERE    (dbo.PMX_MVLI.DestLogUnitIdentKey = {0}) 
		 AND dbo.ORDR.U_PMX_INFC = 'Y'", luid.ToString());


            }

            /// <summary>
            /// Gets the name of the products from stock order by.
            /// </summary>
            /// <param name="gtin">The gtin.</param>
            /// <param name="itemCode">The item code.</param>
            /// <param name="packagingUnit">The packaging unit.</param>
            /// <param name="onlyLogisticCarriers">The only logistic carriers.</param>
            /// <param name="dbTool">The database tool.</param>
            /// <returns></returns>
            public static string GetProducts(string gtin, string itemCode, Produmex.Sbo.Logex.Data.SboCommands.PackagingUnit packagingUnit, bool? onlyLogisticCarriers, DbTool dbTool)
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
                strBuilder.Append("SELECT DISTINCT");

                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemCode.NAME));
                strBuilder.Append(" AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.PRODUCT_CODE.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemName.NAME));
                strBuilder.Append(" AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.PRODUCT_DESCRIPTION.Name));

                //strBuilder.Append(", ");
                //strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxItemTable.NAME, Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxItemTable.Columns.DefaultStorageLocationOrZoneCode.NAME, ProductDataSet.Columns.DEFAULT_LOCATION_CODE.Name));

                strBuilder.Append(", ");
                Produmex.Sbo.Logex.Data.SboCommands.AddBarcodeUsingPackagingUnit(strBuilder, packagingUnit, ProductDataSet.Columns.GTIN.Name);

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxItemTable.Columns.HasBestBeforeDate.NAME));
                strBuilder.Append(" AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.HAS_BEST_BEFORE.Name));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ManBtchNum.NAME));
                strBuilder.Append(" AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(ProductDataSet.Columns.MANBTCHNUM.Name));

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));


                string operand = " WHERE ";
                if (!string.IsNullOrEmpty(gtin))
                {
                    strBuilder.Append(operand);
                    Produmex.Sbo.Logex.Data.SboCommands.AddWhereClauseSearchForBarcode(strBuilder, gtin);
                    operand = " AND ";
                }

                if (!string.IsNullOrEmpty(itemCode))
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemCode.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));
                    operand = " AND ";
                }


                if (onlyLogisticCarriers.HasValue)
                {
                    strBuilder.Append(operand);
                    strBuilder.Append(SqlCommandHelper.EscapeTableName(Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxItemTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables.PmxItemTable.Columns.IsLogisticCarrier.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(onlyLogisticCarriers.Value ? SboBooleanColumnValues.Y.VALUE : SboBooleanColumnValues.N.VALUE, CultureInfo.InvariantCulture));
                    operand = " AND ";

                }

                strBuilder.Append(" ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemName.NAME));
                return strBuilder.ToString();
            }


            /// <summary>
            /// Gets the LUID.
            /// </summary>
            /// <param name="pickListDocEntry">The pick list doc entry.</param>
            /// <returns></returns>
            public static string GetLUID(int pickListDocEntry)
            {

                //SELECT     MAX(dbo.PMX_MVLI.DestLogUnitIdentKey) AS DestLogUnitIdentKey
                //FROM         dbo.PMX_LPLM INNER JOIN
                //                      dbo.PMX_MVLI ON dbo.PMX_LPLM.MoveDocEntry = dbo.PMX_MVLI.DocEntry AND dbo.PMX_LPLM.MoveLineNum = dbo.PMX_MVLI.LineNum
                //WHERE     (dbo.PMX_LPLM.PickListDocEntry = 279)


                StringBuilder strBuilder = new StringBuilder();
                strBuilder.Append("SELECT MAX (");

                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME);
                strBuilder.Append(") AS ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME));

                strBuilder.Append(" FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLinkPickListLastMoveTable.NAME));

                strBuilder.Append(" INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxMoveLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.MoveDocEntry.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.LineNum.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.MoveLineNum.NAME);

                strBuilder.Append(" WHERE ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.PickListDocEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pickListDocEntry, CultureInfo.InvariantCulture));

                return strBuilder.ToString();
            }

        }

        #endregion
    }
}
