﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Collections.ObjectModel;
using System.Data;
using System.Data.Common;
using System.Globalization;
using System.Reflection;
using System.Text;
using System.Transactions;
using System.Xml;
using System.Web;
using System.Net;
using System.IO;
using System.Threading;
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.DataSetClient;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.SlimScreen.WinGui;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.DataObjects;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Generators.PackingController;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Sbo.Logex.MoviantoNL.BusinessObjects;
using Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.MoviantoNL.Providers;


// PmxWorkflowExecutionTypes.SUB_FLOW
namespace Produmex.Sbo.Logex.WorkflowScripts
{
	/// <summary>
	/// Cycle counting
	/// </summary>
	public class WorkflowScript_PalletConsolidationMoviantoScript : WorkflowInstanceScriptBase
	{
        private static class Definitions
        {
            public const string REPORT_PALLETLABEL    = @"MoviantoPalletLabel.rpt";
            public const string REPORT_PALLETMANIFEST = @"MoviantoPalletManifest.rpt";
        }

		// Input parameters
		public ReadOnlyBinder<CultureInfo>   DefaultCultureInfo;
		public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

		private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

		private ISboProviderService m_sboProviderService;


		#region Constructors

		public WorkflowScript_PalletConsolidationMoviantoScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
			: base(parent, factory)
		{

		}

		#endregion

		#region WorkflowInstanceScriptBase Members

		/// <summary>
		/// Executes this instance.
		/// </summary>
		protected override void Execute()
		{
			// Parameters in scope
			Session session = GetScopeParameter("Session") as Session;
			m_sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
			ProdumexError pmxError = null;

			string initialErrorKey = null;
			string deviceID = session.DeviceInstanceId;
			SboUserInfo userInfo = m_sboProviderService.GetCurrentUser();
			string query = null;

            ToolBarButtons toolBarButtons = ToolBarButtons.Back;



            Message msg                = null;


            int     pickListDocEntry   = 0;
            int     salesorderDocEntry = 0;
            string  location           = String.Empty;
            
            string Carrier = String.Empty;
            bool   CMR     = false;
            string AWBNr   = String.Empty;

            string ColliID     = String.Empty;
            string Pallet_SSCC = String.Empty;
            int    Pallet_LUID = 0;

            string buttonKeyPalletConsolidationTask;


        Step_SelectOption:

            msg = null;

            toolBarButtons = ToolBarButtons.Back;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectButtonScreen),
                   DefaultCultureInfo.Get(), BuildParamCollection("InitialErrorKey", initialErrorKey,
                                                                  "TitleKey", "Select Pallet Task",
                                                                  "AllowedButtons", new string[4] { "PALLET LABEL", "SCAN COLLI", "PRINT PALLET MANIFEST", "REMOVE ORDER" },
                                                                  "EnabledToolBarButtons", toolBarButtons));

            msg = WaitForMessage();
            
            initialErrorKey = null;

            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                buttonKeyPalletConsolidationTask = ExtractParameter<string>(msg.Parameters, "buttonKey");

                if (buttonKeyPalletConsolidationTask == "PALLET LABEL")
                {
                    goto Step_SelectCarrier;
                }
                if (buttonKeyPalletConsolidationTask == "SCAN COLLI")
                {
                    ColliID     = String.Empty;
                    Pallet_SSCC = String.Empty;

                    goto Step_ScanColliID;
                }
                if (buttonKeyPalletConsolidationTask == "PRINT PALLET MANIFEST")
                {
                    goto Step_PalletManifest;
                }
                if (buttonKeyPalletConsolidationTask == "REMOVE ORDER")
                {
                    goto Step_RemoveOrder;
                }

                goto Step_End;
            }

            if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_End;
            }


        Step_SelectCarrier:
          
            msg = null;

            query = BuildQuery.GetCarrierList();
            DataSet dsCarrierList = m_sboProviderService.RunView(false, null, null, query);

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectEnterStringValueScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection( "TitleKey"       , "Select Carrier",
                                                                               "StringValueDS"  , dsCarrierList,
                                                                               "InitialErrorKey", null,
                                                                               "AllowToGoBack"  , true,
                                                                               "OnlySelectStringValue", true,
                                                                               "AllowEmptyStringValue", false));

            msg = WaitForMessage();
            initialErrorKey = null;

            if (msg.Name.EndsWith(".StringValueSelected"))
            {
                Carrier = ExtractParameter<string>(msg.Parameters, "stringValue");

                goto Step_AskCMRPrinted;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_SelectOption;
            }

        Step_AskCMRPrinted:

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection( "TitleKey"       , "CMR",
                                                                               "MessageKey"     , "Print CMR?",
                                                                               "ButtonKeyYes"   , "Ja",
                                                                               "ButtonKeyNo"    , "Nee",
                                                                               "AllowStopButton", true));

            msg = WaitForMessage();
            initialErrorKey = null;

            if (msg.Name.EndsWith(".Yes"))
            {
                CMR = true;

                goto Step_CreateSSCC;
            }
            else if (msg.Name.EndsWith(".No"))
            {
                CMR = false;

                goto Step_CreateSSCC;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_SelectCarrier;
            }


        Step_CreateSSCC:

            m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
               delegate(PmxDbConnection conn, object[] parameters)
               {
                    PmxLogisticUnitIDProvider luidProv = new PmxLogisticUnitIDProvider(conn);
                    
                    Pallet_LUID = luidProv.GenerateNewLogisticUnitSSCC(null, out Pallet_SSCC);
                    
                    query = BuildQuery.UpdatePmxLUID(Pallet_LUID, Carrier, CMR);                           
                    SboRecordsetHelper.RunQueryNoReturn(s_log, query, conn);

                    return null;
                });


                goto Step_PrintPalletLabel;
           

        Step_PrintPalletLabel:
           
            try
            {
                m_sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintPalletLabelReport), Pallet_LUID, session.DeviceInstanceId, Definitions.REPORT_PALLETLABEL);
                Thread.Sleep(750);
            }
            catch (Exception ex)
            {
                s_log.Error(ex.ToString());
                throw ex;
            }

            goto Step_SelectOption;


        Step_ScanColliID:

            ColliID     = String.Empty;
            Pallet_SSCC = String.Empty;

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifyBarcodeScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection("TitleKey", "Scan Colli ID",
                                                                               "InitialErrorKey", null,
                                                                               "AllowStopButton", false));

            msg = WaitForMessage();
            initialErrorKey = null;

            if (msg.Name.EndsWith(".DataEntered"))
            {
                ColliID = ExtractParameter<string>(msg.Parameters, "data");

                goto Step_PalletID;
            }
            else if (msg.Name.EndsWith(".StopRequested"))
            {
                goto Step_SelectOption;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {                
                goto Step_SelectOption;
            }

            goto Step_SelectOption;


        Step_PalletID:
            
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifyGS1DataScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "Scan pallet ID",
                "ForceGS1_128", true,
                "AllowStopButton", true,
                "ImageType", GS1ImageType.PalletImage
                ));

            msg = WaitForMessage();

            //clear data
            initialErrorKey = null;

            if (msg.Name.EndsWith(".DataEntered"))
            {
                BarcodeContainer barcodeContainerSSCC = ExtractParameter<BarcodeContainer>(msg.Parameters, "data");
                //check what has been scanned

                if (barcodeContainerSSCC.GS1DataObject != null && !string.IsNullOrEmpty(barcodeContainerSSCC.GS1DataObject.Sscc))
                {
                    //user scanned an SSCC, so we check if it is allowed
                    Pallet_SSCC = barcodeContainerSSCC.GS1DataObject.Sscc;

                    goto Step_CheckColliIDPalletIDValid;
                }


                initialErrorKey = "MSG_ERROR_BARCODE_DOES_NOT_CONTAIN_SSCC";
                goto Step_PalletID;
            }
            else if (msg.Name.EndsWith(".StopRequested"))
            {
                goto Step_SelectOption;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_ScanColliID;
            }

            goto Step_SelectOption;


        Step_CheckColliIDPalletIDValid:

            DataSet dsColliIDCarrier  = m_sboProviderService.RunView(false, null, null, BuildQuery.GetColliIDCarrierQuery(ColliID));
            DataSet dsPalletIDCarrier = m_sboProviderService.RunView(false, null, null, BuildQuery.GetPalletIDCarrierQuery(Pallet_SSCC));

            String ColliIDCarrier  = dsColliIDCarrier.Tables[0].Rows.Count == 0 || dsColliIDCarrier.Tables[0].Rows[0].IsNull(PmxMovPLCDTable.Columns.Carrier.NAME) ? String.Empty : dsColliIDCarrier.Tables[0].Rows[0][PmxMovPLCDTable.Columns.Carrier.NAME].ToString();
            String PalletIDCarrier = dsPalletIDCarrier.Tables[0].Rows.Count == 0 || dsPalletIDCarrier.Tables[0].Rows[0].IsNull("MOV_Carrier") ? String.Empty : dsPalletIDCarrier.Tables[0].Rows[0]["MOV_Carrier"].ToString();

            
            if (ColliIDCarrier == String.Empty || PalletIDCarrier == String.Empty)
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                		   DefaultCultureInfo.Get(), BuildParamCollection(
                			   "InitialErrorKey", null,
                			   "MessageKey", "Carrier is empty.",
                			   "ShowButton", true));

                msg = WaitForMessage();

                goto Step_SelectOption;
            }

            int PmxMovPLCD_InternalKey = Convert.ToInt32(dsColliIDCarrier.Tables[0].Rows[0][PmxMovPLCDTable.Columns.InternalKey.NAME]);
            int PmxPalletID_InternalKey = Convert.ToInt32(dsPalletIDCarrier.Tables[0].Rows[0][PmxLogisticUnitIDTable.Columns.InternalKey.NAME]);
                        
            if (ColliIDCarrier == PalletIDCarrier)
            {
                m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, delegate(PmxDbConnection conn, object[] parameters)
                {
                    PmxMovPLCDProvider prov = new PmxMovPLCDProvider(conn);

                    PmxMovPLCD obj = prov.GetBO(PmxMovPLCD_InternalKey);

                    if (obj != null)
                    {
                        obj.PalletID = PmxPalletID_InternalKey;

                        prov.UpdateBO(obj, false, false);
                    }

                    return null;
                });

                goto Step_ScanColliID;
            }
            else
            {
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                           DefaultCultureInfo.Get(), BuildParamCollection(
                               "InitialErrorKey", null,
                               "MessageKey", "Carriers zijn verschillend.",
                               "ShowButton", true));

                msg = WaitForMessage();

                goto Step_ScanColliID;
            }
          
        
        Step_PalletManifest:

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifyGS1DataScreen),
                                        DefaultCultureInfo.Get(), BuildParamCollection(
                                        "InitialErrorKey", initialErrorKey,
                                        "TitleKey", "Scan pallet ID",
                                        "ForceGS1_128", true,
                                        "AllowStopButton", false,
                                        "ImageType", GS1ImageType.PalletImage
                                        ));

            msg = WaitForMessage();

            //clear data
            initialErrorKey = null;

            if (msg.Name.EndsWith(".DataEntered"))
            {
                BarcodeContainer barcodeContainerSSCC = ExtractParameter<BarcodeContainer>(msg.Parameters, "data");
                //check what has been scanned

                if (barcodeContainerSSCC.GS1DataObject != null && !string.IsNullOrEmpty(barcodeContainerSSCC.GS1DataObject.Sscc))
                {
                    //user scanned an SSCC, so we check if it is allowed
                    Pallet_SSCC = barcodeContainerSSCC.GS1DataObject.Sscc;

                    goto Step_PrintPalletManifest;
                }

                initialErrorKey = "MSG_ERROR_BARCODE_DOES_NOT_CONTAIN_SSCC";
                goto Step_PalletManifest;
            }
            else if (msg.Name.EndsWith(".StopRequested"))
            {
                goto Step_SelectOption;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_SelectOption;
            }

            goto Step_SelectOption;

        Step_PrintPalletManifest:

            string reportPalletManifest = Definitions.REPORT_PALLETMANIFEST;

            try
            {
                DataSet dsPallet = m_sboProviderService.RunView(false, null, null, BuildQuery.GetPalletIDCarrierQuery(Pallet_SSCC));

                if (dsPallet.Tables[0].Rows.Count == 0)
                {
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection(
                                   "InitialErrorKey", null,
                                   "MessageKey", "Could not find Pallet ID.",
                                   "ShowButton", true));

                    msg = WaitForMessage();

                    goto Step_PalletManifest;
                }

                int PalletInternalKey = Convert.ToInt32(dsPallet.Tables[0].Rows[0][PmxLogisticUnitIDTable.Columns.InternalKey.NAME]);

                m_sboProviderService.InvokeMethodWithDbConnection<PmxOsePrinter>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOsePrinter>(ExecutePrintPalletManifestReport), PalletInternalKey, session.DeviceInstanceId, reportPalletManifest);
                Thread.Sleep(750);
            }
            catch (Exception ex)
            {
                s_log.Error(ex.ToString());
                throw ex;
            }

            goto Step_SelectOption;


        Step_RemoveOrder:

            pickListDocEntry   = 0;
            salesorderDocEntry = 0;
            msg = null;

            query = BuildQuery.GetPickList();
            DataSet dsPickList = m_sboProviderService.RunView(false, null, null, query);

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Picking.ISelectPickListScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection("TitleKey", "Select Picklist",
                                                                               "PickListDS", dsPickList,
                                                                               "InitialErrorKey", null));

            msg = WaitForMessage();
            initialErrorKey = null;

            if (msg.Name.EndsWith(".PickListSelected"))
            {
                pickListDocEntry = ExtractParameter<int>(msg.Parameters, "pickListDocEntry");

                DataRow[] rows = dsPickList.Tables[0].Select(string.Format("{0} = {1}", PickListDataSet.Columns.DOC_ENTRY.Name, pickListDocEntry));

                if (rows.Length > 0)
                {
                    salesorderDocEntry = Convert.ToInt32(rows[0]["SalesOrderDocEntry"]);
                    location = Convert.ToString(rows[0]["DestStorLocCode"]);

                    goto Step_ShowCarrierPallets;
                }
                else
                {
                    goto Step_SelectOption;
                }

            }
            else if (msg.Name.EndsWith(".RefreshRequested"))
            {
                goto Step_RemoveOrder;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_SelectOption;
            }

        Step_ShowCarrierPallets:

            query = BuildQuery.GetCarrierPalletsList(pickListDocEntry);
            DataSet dsCarrierPalletList = m_sboProviderService.RunView(false, null, null, query);


            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectSSCCScreen),
                   DefaultCultureInfo.Get(), BuildParamCollection("TitleKey", "Carrier Pallets",
                                                                  "SSCCDS", dsCarrierPalletList,
                                                                  "InitialErrorKey", null,
                                                                  "ShowButtonNoSSCC", false));

            msg = WaitForMessage();
            initialErrorKey = null;

            if (msg.Name.EndsWith(".SSCCSelected"))
            {
                goto Step_RemoveColliIDsConfirmation;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_RemoveOrder;
            }

            goto Step_SelectOption;

        Step_RemoveColliIDsConfirmation:

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                               DefaultCultureInfo.Get(), BuildParamCollection("TitleKey", "Verwijder Order",
                                                                               "MessageKey", "Verwijder Colli's voor Order?",
                                                                               "ButtonKeyYes", "Ja",
                                                                               "ButtonKeyNo", "Nee",
                                                                               "AllowStopButton", true));

            msg = WaitForMessage();
            initialErrorKey = null;

            if (msg.Name.EndsWith(".Yes"))
            {
                CMR = true;

                goto Step_RemoveColliIDs;
            }
            else if (msg.Name.EndsWith(".No"))
            {
                CMR = false;

                goto Step_RemoveOrder;
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_RemoveOrder;
            }

            goto Step_SelectOption;

        Step_RemoveColliIDs:

            m_sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
                delegate(PmxDbConnection conn, object[] parameters)
               {
                    PmxMovPLCDProvider luidProv = new PmxMovPLCDProvider(conn);
                                        
                    query = BuildQuery.GetDeleteCollisQuery(salesorderDocEntry);                           
                    SboRecordsetHelper.RunQueryNoReturn(s_log, query, conn);

                    return null;
                });

            goto Step_SelectOption;

        Step_End:

            pickListDocEntry = 0;
		}

		#endregion


		#region HELPERS

		/// <summary>
		/// Executes the get PMX warehouse info.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxOseWarehouseInfo ExecuteGetPmxWarehouseInfo(PmxDbConnection conn, object[] parameters)
		{
			PmxOseProvider oseProv = new PmxOseProvider(conn);
			return oseProv.GetPmxWarehouseInfo((string)parameters[0]);
		}

		/// <summary>
		/// Gets the number of rows.
		/// </summary>
		/// <param name="ds">The ds.</param>
		/// <returns></returns>
		private int GetNumberOfRows(DataSet ds)
		{
			if (ds != null &&
				ds.Tables.Count > 0)
			{
				return ds.Tables[0].Rows.Count;
			}

			return 0;
		}



		/// <summary>
		/// Executes the get item info.
		/// </summary>
		/// <param name="conn">The conn.</param>
		/// <param name="parameters">The parameters.</param>
		/// <returns></returns>
		private static PmxItemInfo ExecuteGetItemInfo(PmxDbConnection conn, object[] parameters)
		{
			PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
			return prov.GetCachedItemInfo((string)parameters[0]);
		}

        /// <summary>
        /// Executes the report Pallet Label.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOsePrinter ExecutePrintPalletLabelReport(PmxDbConnection conn, object[] parameters)
        {
            PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

            Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
            reportParameters.Add(new ReportParameter("@LUID", (int)parameters[0]));
            
            // TODO : get location
            return prov.PrintReport(reportParameters, (string)parameters[2], 1, (string)parameters[1], null, false);
        }

        /// <summary>
        /// Executes the report Pallet Manifest.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOsePrinter ExecutePrintPalletManifestReport(PmxDbConnection conn, object[] parameters)
        {
            PmxPrintReportProvider prov = new PmxPrintReportProvider(conn);

            Collection<ReportParameter> reportParameters = new Collection<ReportParameter>();
            reportParameters.Add(new ReportParameter("@LUID", (int)parameters[0]));

            return prov.PrintReport(reportParameters, (string)parameters[2], 1, (string)parameters[1], null, false);
        }

        /// <summary>
        /// Executes the report.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static object AddData(PmxDbConnection conn, object[] parameters)
        {
            PmxLogisticUnitIDProvider prov = new PmxLogisticUnitIDProvider(conn);

            using (PmxLogisticUnitID luid = prov.GetNewBO())
            {
                luid.IsFullPallet = true;

                prov.AddBO(luid);
 
            }


            return null;
        }

		#endregion

	}

	#region BUILD QUERY

	/// <summary>
	/// Static class grouping our native queries
	/// </summary>
	internal static class BuildQuery
	{
        /// <summary>
        /// Gets the pick list.
        /// </summary>       
        /// <returns></returns>
        public static string GetCarrierList () 
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.Append("SELECT ");

            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, "@PMX_MOV_CARRI", "Name",  StringValueDataSet.Columns.STRING_VALUE.Name);

            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName("@PMX_MOV_CARRI"));

            return strBuilder.ToString();
        }

        public static string GetColliIDCarrierQuery  (string pColliID)    
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("SELECT ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxMovPLCDTable.NAME, PmxMovPLCDTable.Columns.Carrier.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxMovPLCDTable.NAME, PmxMovPLCDTable.Columns.InternalKey.NAME);
            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxMovPLCDTable.NAME));
            strBuilder.Append(" WHERE ");

            SqlCommandHelper.AddTableColumnName(strBuilder, PmxMovPLCDTable.NAME, PmxMovPLCDTable.Columns.ColliID.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pColliID, CultureInfo.InvariantCulture));

            return strBuilder.ToString();                       
        }
        public static string GetPalletIDCarrierQuery (string pPalletSSCC) 
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("SELECT TOP 1");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, "MOV_Carrier");
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME);
            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));
            strBuilder.Append(" WHERE ");

            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pPalletSSCC, CultureInfo.InvariantCulture));

            strBuilder.Append(" ORDER BY ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME);
            strBuilder.Append(" DESC ");

            return strBuilder.ToString();
        }

        public static string GetDeleteCollisQuery(int pSalesOrderDocEntry)
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("DELETE FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxMovPLCDTable.NAME));
            strBuilder.Append(" WHERE ");

            SqlCommandHelper.AddTableColumnName(strBuilder, PmxMovPLCDTable.NAME, PmxMovPLCDTable.Columns.SalesOrderDocEntry.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(pSalesOrderDocEntry, CultureInfo.InvariantCulture));

            return strBuilder.ToString();
        }

        /// <summary>
        /// Updates the PMX LUID.
        /// </summary>
        /// <param name="luid">The luid.</param>
        /// <param name="carrier">The carrier.</param>
        /// <param name="CMR">CMR.</param>
        /// <returns></returns>
        public static string UpdatePmxLUID(int luid, string carrier, bool CMR)
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("UPDATE ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));

            strBuilder.Append(" SET ");
            //SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.KST_Weight.NAME);
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, "MOV_Carrier");
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(carrier, CultureInfo.InvariantCulture));

            strBuilder.Append(", ");
            //SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTableKST.Columns.KST_Length.NAME);
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, "MOV_CMR");
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(CMR, CultureInfo.InvariantCulture));
            

            strBuilder.Append(" WHERE ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(luid, CultureInfo.InvariantCulture));

            return strBuilder.ToString();
        }

        /// <summary>
        /// Gets the pick list.
        /// </summary>       
        /// <returns></returns>
        public static string GetPickList()
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.Append("SELECT ");
              
            strBuilder.Append(String.Format("{0} AS {1}, "                  , "CardCode"      , PickListDataSet.Columns.CUSTOMER_CODE.Name));
            strBuilder.Append(String.Format("{0} AS {1}, "                  , "CardName"      , PickListDataSet.Columns.CUSTOMER_NAME.Name));
            strBuilder.Append(String.Format("{0} AS {1}, "                  , "DeliveryAdress", PickListDataSet.Columns.CUSTOMER_DELIVERY_ADDRESS.Name));
            strBuilder.Append(String.Format("{0} AS {1}, "                  , "DocEntry"      , PickListDataSet.Columns.DOC_ENTRY.Name));
            strBuilder.Append(String.Format("CAST({0} AS NVARCHAR) AS {1}, ", "DocEntry"      , PickListDataSet.Columns.DOC_ENTRY_AS_STRING.Name));
            strBuilder.Append(String.Format("CAST({0} AS DATETIME) AS {1}, ", "null"          , PickListDataSet.Columns.DUE_DATE.Name));
            strBuilder.Append(String.Format("CAST({0} AS NVARCHAR) AS {1}, ", "null"          , PickListDataSet.Columns.DEST_STORLOC_CODE.Name));
            strBuilder.Append(String.Format("CAST({0} AS NVARCHAR) AS {1}, ", "null"          , PickListDataSet.Columns.PICK_PACK_REMARKS.Name));
            strBuilder.Append(String.Format("CAST({0} AS NVARCHAR) AS {1}, ", "null"          , PickListDataSet.Columns.MOVE_TO_WAREHOUSE_CODE.Name));
            strBuilder.Append(String.Format("CAST({0} AS NVARCHAR) AS {1}, ", "null"          , PickListDataSet.Columns.MOVE_TO_WAREHOUSE_NAME.Name));
            strBuilder.Append(String.Format("CAST({0} AS NVARCHAR) AS {1}, ", "null"          , PickListDataSet.Columns.SHIPPING_ID.Name));

            strBuilder.Append(String.Format("{0},"                          , "SalesOrderDocEntry"));
            strBuilder.Append(String.Format("{0},"                          , "DestStorLocCode"));
            strBuilder.Append(String.Format("{0},"                          , "ContactName"));
            strBuilder.Append(String.Format("{0},"                          , "ContactTel"));
            strBuilder.Append(String.Format("{0},"                          , "ContactFax"));
            strBuilder.Append(String.Format("{0}"                           , "ContactCellolar"));
            
            strBuilder.Append("\r\n FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName("PMX_MOV_PACKED_PICKLISTS"));

            return strBuilder.ToString();
        }

        public static string GetCarrierPalletsList(int picklistDocEntry)
        {
            StringBuilder strBuilder = new StringBuilder();

            strBuilder.Append("SELECT ");

            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxMovPLCDTable.NAME, PmxMovPLCDTable.Columns.Carrier.NAME, SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME, SSCCDataSet.Columns.LUID.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME, SSCCDataSet.Columns.SSCC.Name);
            strBuilder.Append(", ");

            SqlCommandHelper.AddStringNullColumnWithName(strBuilder, SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddStringNullColumnWithName(strBuilder, SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Name);


            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxMovPLCDTable.NAME));
            strBuilder.Append(" JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxMovPLCDTable.NAME, PmxMovPLCDTable.Columns.PalletID.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME);
            strBuilder.Append(" WHERE ");

            SqlCommandHelper.AddTableColumnName(strBuilder, PmxMovPLCDTable.NAME, PmxMovPLCDTable.Columns.PickListDocEntry.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(picklistDocEntry, CultureInfo.InvariantCulture));

            return strBuilder.ToString();            
        }
	}


	#endregion

}
