// TOAN 20180528 Switch USER_ID on XML for ONT
//css_ref Produmex.Foundation.SlimScreen.Interfaces;
//css_ref Produmex.Foundation.Wwf.Sbo;
//css_ref Produmex.Foundation.Data.dll;
//css_ref Produmex.Foundation.Data.Sbo.dll;
//css_ref Produmex.Foundation.dll;
//css_ref Produmex.Foundation.Sbo.dll;
//css_ref Produmex.Sbo.Logex.Data.dll;
//css_ref Produmex.Sbo.Logex.MoviantoNL.dll;

using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Text;
using Produmex.Foundation.Convertors;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Providers;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Foundation;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.Utilities;
using System.Xml;
using Produmex.Foundation.Collections;

// PmxWorkflowExecutionTypes.HOOK_FLOW
namespace WorkflowScript_SalesDeliveryCreatedHookScript
{
    /// <summary>
    /// This is a hook script, meant to be customized on a "per project" basis
    /// </summary>
    public class WorkflowScript_SalesDeliveryCreatedHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        private struct Definitions
        {
            /// <summary>
            /// 
            /// </summary>
            public const string EXPORT_PATH = @"\\snlosspwfs01\InterfacePrince\To_SiS";
            //public const string EXPORT_PATH = @"\\192.168.10.18\prd";

            /// <summary>
            /// Password
            /// </summary>
            public const string SHORT_NAME = "manager";

            /// <summary>
            /// Password
            /// </summary>
            public const string PASSWORD = "UploadMoviantoSAP";

            /// <summary>
            /// User id
            /// </summary>
            public const string USER_ID = "Upload@moviantobenelux.com";

            /// <summary>
            /// Company code
            /// </summary>
            public const string COMPANY_CODE = "151";

            /// <summary>
            /// Agent building
            /// </summary>
            public const string AGENT_BUILDING_COVIDIEN = "Covidien";

            /// <summary>
            /// Agent building
            /// </summary>
            public const string COVIDIEN_CARD_CODE = "103003";

            /// <summary>
            /// Agent building
            /// </summary>
            public const string AGENT_BUILDING_NOT_COVIDIEN = "Movianto";

            /// <summary>
            /// Ship Type Name Like
            /// </summary>
            public const string SHIP_TYPE_NAME_LIKE = "SIS";
        }

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
        public ReadOnlyBinder<DocumentRef[]> DocumentRefs;

        // Output parameters
        // <none>

        // Sub-flows
        // <none>

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_SalesDeliveryCreatedHookScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_SalesDeliveryCreatedHookScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            Message msg = null;

            string query = null;
            ProdumexError pmxError = null;
            string msgKey = null;

            DocumentRef[] documentRefs = DocumentRefs.Get();

            List<int> listOfSalesDeliveries = new List<int>();

            foreach (DocumentRef docRef in documentRefs)
            {
                if (docRef.DocType == SboObjectTypes.SalesDelivery)
                {
                    listOfSalesDeliveries.Add(docRef.DocEntry);
                }
            }

            if (listOfSalesDeliveries.Count > 0)
            {
                try
                {
                    sboProviderService.InvokeMethodWithDbConnection<bool>(false, false, null, null, CreateXMLFile, session.DeviceInstanceId, listOfSalesDeliveries);
                }
                catch (ProdumexException ex)
                {
                    s_log.Error("", ex);
                    msgKey = ExceptionHelper.Format(ex, this.DefaultCultureInfo);
                    pmxError = ex.Error;
                }
                catch (Exception ex)
                {
                    s_log.Error("", ex);
                    msgKey = ExceptionHelper.Format(ex, this.DefaultCultureInfo);
                }

                if (msgKey != null)
                {
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "InitialErrorKey", null,
                            "MessageKey", msgKey,
                            "PmxError", pmxError,
                            "ShowButton", true));
                    WaitForMessage();
                }
            }
        }

        #endregion

        #region HELPERS

        /// <summary>
        /// Creates the XML file.
        /// </summary>
        /// <param name="dbConn">The db conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private bool CreateXMLFile(PmxDbConnection dbConn, object[] parameters)
        {
			
            string deviceInstanceId = (string)parameters[0];

            List<int> listOfSalesDeliveries = (List<int>)parameters[1];

			s_log.DebugFormat("CreateXMLFile for deliveries {0}", SqlCommandHelper.FormatColumnValue(listOfSalesDeliveries, dbConn.DbCultureInfo));

			PmxParameterValueProvider parProv = new PmxParameterValueProvider(dbConn);
            PmxDictionary<string, PmxParameterValue> upsParameters = parProv.GetParameterValuesAsDictionary(String.Format("{0}->{1}", deviceInstanceId, PmxPropertyCodes.OSE_PARAMETERSET), true, false);

            //PmxSalesDeliveryNoteProvider delNoteProv = new PmxSalesDeliveryNoteProvider((SboConnection)dbConn);

            // Get configuration
            PmxConfigProvider cfgProv = new PmxConfigProvider(dbConn);
            PmxConfig cfg = cfgProv.GetBO(PmxConfigCodes.INTERFACE_MONITOR_OUT_PATH);

            PmxConvertorProvider cnvProv = new PmxConvertorProvider(dbConn);
            IConvertor convertor = cnvProv.GetConvertor(cfg.ConvertorCode);
            string outputDir = Convert.ToString(convertor.ConvertStringToObject(cfg.ConfigValue));

            //CultureInfo englishCulture = new CultureInfo("en-US");

            outputDir = Definitions.EXPORT_PATH;

            foreach (DocumentRef docRef in DocumentRefs.Get())
            {
                DateTime now = DateTime.Now;
                string tempFileName = String.Format("LCD_{0}_{1:yyyyMMddHHmmssff}.XML.new", docRef.DocEntry, now);
                string fileName = String.Format("LCD_{0}_{1:yyyyMMddHHmmssff}.XML", docRef.DocEntry, now);

                string query = BuildQuery.GetSboSalesDeliveryHeader(docRef.DocEntry, Definitions.SHIP_TYPE_NAME_LIKE, dbConn.DbCultureInfo);

				s_log.DebugFormat("CreateXMLFile Query executed = {0}", query);

				using (ISboRecordset rsOrder = SboRecordsetHelper.RunQuery(s_log, query, dbConn))
                {
					if (rsOrder.EoF)
					{
						s_log.Debug("CreateXMLFile Query returned no rows, so NO EXPORT");
						continue;  // Skip if not needed to export
					}

                    // Build up the XML document
                    XmlDocument xmlDoc = new XmlDocument();

                    XmlDeclaration declaration = xmlDoc.CreateXmlDeclaration("1.0", "UTF-8", null);
                    xmlDoc.AppendChild(declaration);

                    string cardCodeInitiator = rsOrder.GetTypedValue<string>(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesDeliveryTable.Columns.InitiatorCode.NAME);
                    //  string cardCode = rsOrder.GetTypedValue<string>(SboSalesDeliveryNoteTable.Columns.CardCode.NAME);
                    bool isCovidien = false;

                    XmlElement rootElm = xmlDoc.CreateElement("insert_shipment");

                    //XmlAttribute attr = xmlDoc.CreateAttribute("shortname");
                    //attr.Value = Definitions.SHORT_NAME;
                    //rootElm.Attributes.Append(attr);

                    XmlAttribute attr = xmlDoc.CreateAttribute("code_company");
                    attr.Value = Definitions.COMPANY_CODE;
                    rootElm.Attributes.Append(attr);

                    attr = xmlDoc.CreateAttribute("userid");
                    if (cardCodeInitiator == "115001") // TOAN 20180528 Switch USER_ID on XML for ONT
                    {
                        attr.Value = "ONT.Upload@moviantobenelux.com";
                    }
                    else
                    {
                        attr.Value = Definitions.USER_ID;
                    }
                    rootElm.Attributes.Append(attr);

                    attr = xmlDoc.CreateAttribute("password");
                    attr.Value = Definitions.PASSWORD;
                    rootElm.Attributes.Append(attr);

                    attr = xmlDoc.CreateAttribute("agent_building");
                    attr.Value = cardCodeInitiator;

                    //if (cardCodeInitiator.ToUpper() == Definitions.COVIDIEN_CARD_CODE.ToUpper())
                    //{
                    //    isCovidien = true;
                    //    attr.Value = Definitions.AGENT_BUILDING_COVIDIEN;
                    //}
                    //else
                    //{
                    //    attr.Value = Definitions.AGENT_BUILDING_NOT_COVIDIEN;
                    //}
                    rootElm.Attributes.Append(attr);

                    //get line data
                    bool isTemperatureControlled = false;

                    int? pickListDocEntry = null;
                    query = BuildQuery.GetSboSalesDeliveryLine(docRef.DocEntry, dbConn.DbCultureInfo);
                    using (ISboRecordset rsOrderLine = SboRecordsetHelper.RunQuery(s_log, query, dbConn))
                    {
                        while (!rsOrderLine.EoF)
                        {
                            if (!pickListDocEntry.HasValue)
                            {
                                pickListDocEntry = rsOrderLine.GetTypedValue<int?>(PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.PickListLineDocEntry.NAME);
                            }

                            if (rsOrderLine.GetTypedValue<string>(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxItemTable.Columns.Categorie.NAME) == "Y")
                            {
                                //De vlag temperature controlled is de UDF OITM.U_PMX_DR_CATEGORIE.
                                //Deze UDF heeft de volgende waardes: Standaard (1),  Cytostatica (2) en Contrast Media (3)
                                //Wanneer de UDF Contrast Media is (Dus de waarde 3), dan is dit artikel temperature controlled.
                                isTemperatureControlled = true;
                                break;
                            }
                            rsOrderLine.MoveNext();
                        }
                    }

                    xmlDoc.AppendChild(rootElm);
                    {
                        XmlElement dataElm = xmlDoc.CreateElement("action");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode("STORE"));

                        dataElm = xmlDoc.CreateElement("reference");
                        rootElm.AppendChild(dataElm);
                        if (pickListDocEntry.HasValue)
                        {
                            dataElm.AppendChild(xmlDoc.CreateTextNode(cardCodeInitiator + pickListDocEntry.Value.ToString()));
                        }
                        else
                        {
                            dataElm.AppendChild(xmlDoc.CreateTextNode("No Pick list"));
                        }

                        string account = "00001";

                        if (isCovidien)
                        {
                            //check if it is temperature controller
                            if (isTemperatureControlled)
                            {
                                account = "DPT-10";
                            }
                            else
                            {
                                account = "DPT-9";
                            }
                        }

                        dataElm = xmlDoc.CreateElement("account");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(account));

                        dataElm = xmlDoc.CreateElement("address11");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(PmxSalesDeliveryNoteTable.Columns.CardName.NAME), 50)));

                        string street = string.Format("{0} {1}", rsOrder.GetTypedValue<string>(SboSalesDeliveryTaxExtensionTable.Columns.StreetS.NAME),
                            rsOrder.GetTypedValue<string>(SboSalesDeliveryTaxExtensionTable.Columns.StreetNoS.NAME));

                        dataElm = xmlDoc.CreateElement("address12");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(street, 50)));

                        //if (street.Length > 50)
                        //{
                        //    //if not everything is on first line, create second line
                        //    dataElm = xmlDoc.CreateElement("address13");
                        //    rootElm.AppendChild(dataElm);
                        //    dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(street.Substring(50, street.Length - 50), 50)));
                        //}

                        dataElm = xmlDoc.CreateElement("address14");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(SboSalesDeliveryTaxExtensionTable.Columns.CityS.NAME), 50)));

                        //dataElm = xmlDoc.CreateElement("address2");
                        dataElm = xmlDoc.CreateElement("address13");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.Address2.NAME), 100)));

                        //dataElm = xmlDoc.CreateElement("name2");
                        dataElm = xmlDoc.CreateElement("deliver");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.Name2.NAME), 100)));

                        dataElm = xmlDoc.CreateElement("code_country_to");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(SboSalesDeliveryTaxExtensionTable.Columns.CountryS.NAME), 2)));

                        dataElm = xmlDoc.CreateElement("postcode_delivery");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(SboSalesDeliveryTaxExtensionTable.Columns.ZipCodeS.NAME), 15)));

                        dataElm = xmlDoc.CreateElement("delivery");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(SboContactPersonTable.Columns.Name.NAME), 30)));

                        dataElm = xmlDoc.CreateElement("deliver_fax");
                        rootElm.AppendChild(dataElm);

                        string deliver_fax = rsOrder.GetTypedValue<string>(SboBusinessPartnerTable.Columns.Phone1.NAME);

                        if (string.IsNullOrEmpty(deliver_fax))
                        {
                            //Take other value
                            deliver_fax = rsOrder.GetTypedValue<string>(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField1.NAME);
                        }

                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(deliver_fax, 20)));

                        dataElm = xmlDoc.CreateElement("carrier_name");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(""));

                        //Service level
                        string serviceLevelCode = rsOrder.GetTypedValue<string>(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesDeliveryTable.Columns.ServiceLevel.NAME);
                        string serviceLevel = null;

                        //- als er geen code is dan is <service_level> leeg
                        //- als code ND, dan heeft <service_level> de waarde EXPRESS
                        //- als code BN, dan heeft <service_level> de waarde 1200
                        //- als code N2, dan heeft <service_level> de waarde 1000
                        //- als code N1, dan heeft <service_level> de waarde 0800
                        switch (serviceLevelCode)
                        {
                            case "ND":
                                serviceLevel = "EXPRESS";
                                break;
                            case "BN":
                                serviceLevel = "1200";
                                break;
                            case "N2":
                                serviceLevel = "1000";
                                break;
                            case "N1":
                                serviceLevel = "0800";
                                break;
                            default:
                                serviceLevel = "";
                                break;
                        }

                        dataElm = xmlDoc.CreateElement("service_level");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(serviceLevel));

                        //dataElm = xmlDoc.CreateElement("deliver_fax");
                        //rootElm.AppendChild(dataElm);
                        //dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(SboBusinessPartnerTable.Columns.Phone1.NAME), 20)));

                        dataElm = xmlDoc.CreateElement("weight");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode("0"));

                        XmlElement referenceList = xmlDoc.CreateElement("reference_list");
                        rootElm.AppendChild(referenceList);
                        {
                            dataElm = xmlDoc.CreateElement("reference_tag");
                            referenceList.AppendChild(dataElm);
                            dataElm.AppendChild(xmlDoc.CreateTextNode("lieferschein"));

                            dataElm = xmlDoc.CreateElement("reference_value");
                            referenceList.AppendChild(dataElm);
                            dataElm.AppendChild(xmlDoc.CreateTextNode(rsOrder.GetTypedValue<int>(PmxSalesDeliveryNoteTable.Columns.DocNum.NAME).ToString()));

                            dataElm = xmlDoc.CreateElement("reference_tag");
                            referenceList.AppendChild(dataElm);
                            dataElm.AppendChild(xmlDoc.CreateTextNode("customer_po_nr"));

                            dataElm = xmlDoc.CreateElement("reference_value");
                            referenceList.AppendChild(dataElm);
                            dataElm.AppendChild(xmlDoc.CreateTextNode(rsOrder.GetTypedValue<string>(PmxSalesDeliveryNoteTable.Columns.NumAtCard.NAME)));
                        }

                        dataElm = xmlDoc.CreateElement("TextField1");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField1.NAME), 50)));

                        dataElm = xmlDoc.CreateElement("TextField2");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField2.NAME), 50)));

                        dataElm = xmlDoc.CreateElement("TextField3");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField3.NAME), 50)));

                        dataElm = xmlDoc.CreateElement("TextField4");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField4.NAME), 50)));

                        dataElm = xmlDoc.CreateElement("TextField5");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField5.NAME), 50)));

                        dataElm = xmlDoc.CreateElement("TextField6");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField6.NAME), 50)));

                        dataElm = xmlDoc.CreateElement("TextField7");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField7.NAME), 50)));

                        dataElm = xmlDoc.CreateElement("TextField8");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField8.NAME), 50)));

                        dataElm = xmlDoc.CreateElement("TextField9");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField9.NAME), 50)));

                        dataElm = xmlDoc.CreateElement("TextField10");
                        rootElm.AppendChild(dataElm);
                        dataElm.AppendChild(xmlDoc.CreateTextNode(StringHelper.Truncate(rsOrder.GetTypedValue<string>(Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField10.NAME), 50)));
                    }

                    // Write XML file with encoding "Windows-1252"

                    Encoding Utf8 = new UTF8Encoding(false);

                    System.Xml.XmlWriterSettings settings = new System.Xml.XmlWriterSettings();
                    settings.Encoding = Utf8;//Encoding.UTF8;//.GetEncoding(1252);  // This is "Windows-1252"
                    settings.Indent = true;
                    settings.IndentChars = "    ";
                    settings.NewLineChars = "\r\n";
                    using (System.Xml.XmlWriter xmlWriter = System.Xml.XmlWriter.Create(System.IO.Path.Combine(outputDir, tempFileName), settings))
                    {
                        xmlDoc.Save(xmlWriter);
                    }

                    System.IO.File.Move(System.IO.Path.Combine(outputDir, tempFileName), System.IO.Path.Combine(outputDir, fileName));

					s_log.DebugFormat("CreateXMLFile File created: {0}", System.IO.Path.Combine(outputDir, fileName));
				}
            }

            return true;
        }

        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null && ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }
            return 0;
        }

        #endregion

        #region QUERY

        private class BuildQuery
        {
            /// <summary>
            /// Gets the sbo sales delivery header.
            /// </summary>
            /// <param name="docEntry">The doc entry.</param>
            /// <param name="shipTypeNameLike">The ship type name like.</param>
            /// <param name="cultInfo">The cult info.</param>
            /// <returns></returns>
            internal static string GetSboSalesDeliveryHeader(int docEntry, string shipTypeNameLike, CultureInfo cultInfo)
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat("SELECT {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.DocNum.NAME));
                strBuilder.AppendFormat(", {0}",
                  SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.CardCode.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.CardName.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.NumAtCard.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryTaxExtensionTable.NAME, SboSalesDeliveryTaxExtensionTable.Columns.StreetS.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryTaxExtensionTable.NAME, SboSalesDeliveryTaxExtensionTable.Columns.StreetNoS.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryTaxExtensionTable.NAME, SboSalesDeliveryTaxExtensionTable.Columns.ZipCodeS.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryTaxExtensionTable.NAME, SboSalesDeliveryTaxExtensionTable.Columns.CityS.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryTaxExtensionTable.NAME, SboSalesDeliveryTaxExtensionTable.Columns.StateS.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryTaxExtensionTable.NAME, SboSalesDeliveryTaxExtensionTable.Columns.CountryS.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.Phone1.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.DocCurrency.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.DocTotal.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboContactPersonTable.NAME, SboContactPersonTable.Columns.Name.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesDeliveryTable.Columns.ServiceLevel.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesDeliveryTable.Columns.InitiatorCode.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.Address2.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.Name2.NAME));
                strBuilder.AppendFormat(", {0}",
                   SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField1.NAME));
                strBuilder.AppendFormat(", {0}",
                 SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField2.NAME));
                strBuilder.AppendFormat(", {0}",
                 SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField3.NAME));
                strBuilder.AppendFormat(", {0}",
                 SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField4.NAME));
                strBuilder.AppendFormat(", {0}",
                 SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField5.NAME));
                strBuilder.AppendFormat(", {0}",
                 SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField6.NAME));
                strBuilder.AppendFormat(", {0}",
                 SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField7.NAME));
                strBuilder.AppendFormat(", {0}",
                 SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField8.NAME));
                strBuilder.AppendFormat(", {0}",
                 SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField9.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxSalesOrderTable.Columns.TextField10.NAME));

                strBuilder.AppendFormat("\r\nFROM {0}",
                    SqlCommandHelper.EscapeTableName(SboSalesDeliveryNoteTable.NAME));

                strBuilder.AppendFormat("\r\nINNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(SboSalesDeliveryTaxExtensionTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryTaxExtensionTable.NAME, SboSalesDeliveryTaxExtensionTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.DocEntry.NAME));

                strBuilder.AppendFormat("\r\nINNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(SboBusinessPartnerTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(SboBusinessPartnerTable.NAME, SboBusinessPartnerTable.Columns.CardCode.NAME),
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.CardCode.NAME));

                strBuilder.AppendFormat("\r\nLEFT JOIN {0}",
                    SqlCommandHelper.EscapeTableName(SboPaymentTermsTypeTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(SboPaymentTermsTypeTable.NAME, SboPaymentTermsTypeTable.Columns.GroupNum.NAME),
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.PaymentTermsGroupNum.NAME));

                strBuilder.AppendFormat("\r\nLEFT JOIN {0}",
                    SqlCommandHelper.EscapeTableName(SboContactPersonTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(SboContactPersonTable.NAME, SboContactPersonTable.Columns.CntctCode.NAME),
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.CntctCode.NAME));

                strBuilder.AppendFormat("\r\nWHERE {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(docEntry, cultInfo));
                strBuilder.AppendFormat(" AND {0} IN (",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteTable.NAME, SboSalesDeliveryNoteTable.Columns.TrnspCode.NAME));
                {
                    strBuilder.AppendFormat("    SELECT {0}",
                        SqlCommandHelper.EscapeTableColumnName(SboShippingTypeTable.NAME, SboShippingTypeTable.Columns.Code.NAME));
                    strBuilder.AppendFormat("\r\n    FROM {0}",
                        SqlCommandHelper.EscapeTableName(SboShippingTypeTable.NAME));
                    strBuilder.AppendFormat("\r\n    WHERE ( {0} LIKE '{1}%'",
                        SqlCommandHelper.EscapeTableColumnName(SboShippingTypeTable.NAME, SboShippingTypeTable.Columns.Name.NAME), shipTypeNameLike);
                    strBuilder.AppendFormat("\r\n    OR {0} LIKE '%{1}'",
                        SqlCommandHelper.EscapeTableColumnName(SboShippingTypeTable.NAME, SboShippingTypeTable.Columns.Name.NAME), shipTypeNameLike);
                    strBuilder.AppendFormat("\r\n    OR {0} LIKE '%{1}%' )",
                        SqlCommandHelper.EscapeTableColumnName(SboShippingTypeTable.NAME, SboShippingTypeTable.Columns.Name.NAME), shipTypeNameLike);

                    strBuilder.AppendFormat("\r\n    AND OSHP.U_PMX_AUSH = 'Y'");
                }
                strBuilder.Append(" )");

                return strBuilder.ToString();
            }

            /// <summary>
            /// Gets the sbo sales delivery line.
            /// </summary>
            /// <param name="docEntry">The doc entry.</param>
            /// <param name="cultInfo">The cult info.</param>
            /// <returns></returns>
            internal static string GetSboSalesDeliveryLine(int docEntry, CultureInfo cultInfo)
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat("SELECT {0}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteLineTable.NAME, SboSalesDeliveryNoteLineTable.Columns.ItemCode.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(PmxItemTable.NAME, Produmex.Sbo.Logex.MoviantoNL.BusinessObjects.Definitions.Tables.PmxItemTable.Columns.Categorie.NAME));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnName(PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME, PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.PickListLineDocEntry.NAME));

                strBuilder.AppendFormat("\r\nFROM {0}",
                    SqlCommandHelper.EscapeTableName(SboSalesDeliveryNoteLineTable.NAME));

                strBuilder.AppendFormat("\r\nINNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxItemTable.NAME, PmxItemTable.Columns.ItemCode.NAME),
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteLineTable.NAME, SboSalesDeliveryNoteLineTable.Columns.ItemCode.NAME));

                strBuilder.AppendFormat("\r\nLEFT JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME, PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.SalesDeliveryNoteLineDocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteLineTable.NAME, SboSalesDeliveryNoteLineTable.Columns.DocEntry.NAME));
                strBuilder.AppendFormat(" AND {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxLinkSalesDeliveryNoteLinePickListLineTable.NAME, PmxLinkSalesDeliveryNoteLinePickListLineTable.Columns.SalesDeliveryNoteLineLineNum.NAME),
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteLineTable.NAME, SboSalesDeliveryNoteLineTable.Columns.LineNum.NAME));

                strBuilder.AppendFormat("\r\nWHERE {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(SboSalesDeliveryNoteLineTable.NAME, SboSalesDeliveryNoteLineTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(docEntry, cultInfo));

                return strBuilder.ToString();
            }
        }

        #endregion
    }
}